#####################	SETS
param NM>=1, integer ;
param NG>=1, integer ;
param nk1>=1, integer ;
param nk2>=1, integer ;
param nk3>=1, integer ;
param nk6>=1, integer ;
param nk7>=1, integer ;
param NK1:= 2^nk1, integer ;	## numero de tramos
param NK2:= 2^nk2, integer ;
param NK3:= 2^nk3, integer ;
param NK6:= 2^nk6, integer ;
param NK7:= 2^nk7, integer ;

set PRODUCTS ;
set HOSTS ;
## Etapas
set STAGES1 ;		## etapas batch
set STAGES2 ;		## etapas semicontinuas
set STAGES3 ;		## etapas cromatograficas
set STAGES4 ;		## ultima etapa cromatografica
set STAGES5:= STAGES3 union STAGES4 ;
set STAGES:= STAGES1 union STAGES2 union STAGES3 union STAGES4 ;
## unidades
set UNIT1:= STAGES1 union STAGES2 union STAGES3 union STAGES4 ;
set UNIT2 within {STAGES2 union STAGES3 union STAGES4} ;			# etapas con tanque de permeado
set UNIT3:= STAGES2 union STAGES3 union STAGES4 ;
## routes
set PROUTES within { PRODUCTS , HOSTS , STAGES } ;
set ROUTES = setof {(i,h,j) in PROUTES} (h,j) ;
set ROUTESP within { HOSTS , UNIT2 } ;
set ROUTES2 = setof {(h,j) in ROUTESP: j in STAGES2} (h,j) ;
## i,h
set HP within {PRODUCTS , HOSTS} ;
## unidades en paralelo
set UNITSM:= 1..NM ordered ;
set UNITSG:= 1..NG ordered ;
## transformaciones MIP
set TMIP1U:= 1..NK1+1 ;		## conjunto de puntos
set TMIP2U:= 1..NK2+1 ;
set TMIP3U:= 1..NK3+1 ;
set TMIP6U:= 1..NK6+1 ;
set TMIP7U:= 1..NK7+1 ;

##### PARAMETERS
## Coeficientes de costo
param c1{j in UNIT1} ;
param c2{j in UNIT2} ;
param c3{j in UNIT3} ;
param cmax := max(max{k in UNIT1} c1[k],max{k in UNIT2} c2[k],max{k in UNIT3} c3[k]) ;
param C1{j in UNIT1} := c1[j]/(cmax) ;
param C2{j in UNIT2} := c2[j]/(cmax) ;
param C3{j in UNIT3} := c3[j]/(cmax) ;
param gamma1{j in UNIT1} ;
param gamma2{j in UNIT2} ;
param gamma3{j in UNIT3} ;

## Factores de tamaño y tiempo
param S1{(i,h,j) in PROUTES: j in UNIT1} ;
param S2{(i,h,j) in PROUTES: (h,j) in ROUTESP} ;
param S3{(i,h,j) in PROUTES: j in STAGES5} ;
param s1{(i,h,j) in PROUTES: j in UNIT1}				:= log(S1[i,h,j]) ;
param s2{(i,h,j) in PROUTES: (h,j) in ROUTESP}	:= log(S2[i,h,j]) ;
param s3{(i,h,j) in PROUTES: j in STAGES5}			:= log(S3[i,h,j]) ;
param T0{(i,h,j) in PROUTES: j in STAGES1 union STAGES5} ;
param T1{(i,h,j) in PROUTES: j in STAGES2 union STAGES3} ;
param t0{(i,h,j) in PROUTES: j in STAGES1 union STAGES5} := log(T0[i,h,j]) ;
param t1{(i,h,j) in PROUTES: j in STAGES2 union STAGES3} := log(T1[i,h,j]) ;

## coeficientes de horizonte de produccion
param d{i in PRODUCTS} ;
param delta ;
param coef_delta ;
param rho ;
param NR ;
param RHO := rho/cmax ;

## limites de variables
param y1LO{j in UNIT1} ;
param y1UP{j in UNIT1} ;
param y2LO{j in UNIT2} ;
param y2UP{j in UNIT2} ;
param y3LO{j in UNIT3} ;
param y3UP{j in UNIT3} ;
param y4LO{i in PRODUCTS} ;
param y4UP{i in PRODUCTS} ;
param y5LO{i in PRODUCTS} ;
param y5UP{i in PRODUCTS} ;

param y1min{j in UNIT1} ;
param y2min{j in UNIT2} ;
param y3min{j in UNIT3} ;
param y4min{i in PRODUCTS} ;
param y5min ;

param y1lo{j in UNIT1} ;
param y1up{j in UNIT1} ;
param y2lo{j in UNIT2} ;
param y2up{j in UNIT2} ;
param y3lo{j in UNIT3} ;
param y3up{j in UNIT3} ;

param y4lo{(i,h) in HP} ;
param y4up{(i,h) in HP} ;

param y5lo{(i,h) in HP} ;
param y5up{(i,h) in HP} ;

param z1lo{j in UNIT1} ;
param z1up{j in UNIT1} ;
param z2lo{j in UNIT2} ;
param z2up{j in UNIT2} ;
param z3lo{j in UNIT3} ;
param z3up{j in UNIT3} ;

param z6lo{(i,h,j) in PROUTES: j in STAGES3} ;
param z6up{(i,h,j) in PROUTES: j in STAGES3} ;

param z7lo{(i,h) in HP} ;
param z7up{(i,h) in HP} ;

param v1lo{j in UNIT1} ;
param v1up{j in UNIT1} ;
param v2lo{j in UNIT2} ;
param v2up{j in UNIT2} ;
param v3lo{j in UNIT3} ;
param v3up{j in UNIT3} ;
param v7lo{(i,h) in HP} ;
param v7up{(i,h) in HP} ;

## constantes para transformaciones MIP
set LIST ordered ;
set LISTleft  ordered ;
set LISTright ordered ;
set LISTleftaux ordered ;
set LISTrightaux ordered ;
set LISTaux ordered ;
param lo ;
param up ;
param new ;

param a1{j in UNIT1} := (z1up[j] - z1lo[j])/NK1 ;
param b1{j in UNIT1 , k in TMIP1U} ;
param beta1{j in UNIT1 , k in TMIP1U} ;
param alpha1{j in UNIT1 , k in TMIP1U} ;
param slope1{j in UNIT1 , k in TMIP1U} ;

param a2{j in UNIT2} := (z2up[j] - z2lo[j])/NK2 ;
param b2{j in UNIT2, k in TMIP2U} ;
param beta2{j in UNIT2, k in TMIP2U} ;
param alpha2{j in UNIT2, k in TMIP2U} ;
param slope2{j in UNIT2, k in TMIP2U} ;

param a3{j in UNIT3} := (z3up[j] - z3lo[j])/NK3 ;
param b3{j in UNIT3, k in TMIP3U} ;
param beta3{j in UNIT3, k in TMIP3U} ;
param alpha3{j in UNIT3, k in TMIP3U} ;
param slope3{j in UNIT3, k in TMIP3U} ;

param a6{(i,h,j) in PROUTES:j in STAGES3} := (z6up[i,h,j] - z6lo[i,h,j])/NK6 ;
param b6{(i,h,j) in PROUTES, k in TMIP6U: j in STAGES3} ;
param beta6{(i,h,j) in PROUTES, k in TMIP6U: j in STAGES3} ;
param alpha6{(i,h,j) in PROUTES, k in TMIP6U: j in STAGES3} ;
param slope6{(i,h,j) in PROUTES, k in TMIP6U: j in STAGES3} ;

param a7{(i,h) in HP} := (z7up[i,h] - z7lo[i,h])/NK7 ;
param b7{(i,h) in HP, k in TMIP7U} ;
param beta7{(i,h) in HP, k in TMIP7U} ;
param alpha7{(i,h) in HP, k in TMIP7U} ;
param slope7{(i,h) in HP, k in TMIP7U} ;

##### VARIABLES
var r>= 0 , <= NR ;
var y1{j in UNIT1}	;
var y2{j in UNIT2}	;
var y3{j in UNIT3}	;
var y4{(i,h) in HP}	;
var y5{(i,h) in HP}	;
var y6{j in STAGES, k in UNITSG} binary , >= 0 , <= 1 ;
var y7{j in STAGES, k in UNITSM} binary , >= 0 , <= 1 ;
var x1{j in STAGES} ;
var x2{j in STAGES} ;
var v1{j in UNIT1} 	;
var v2{j in UNIT2} 	;
var v3{j in UNIT3} 	;
var v7{(i,h) in HP} ;
var z1{(i,h) in HP} binary , >= 0 , <= 1 ;
var z2{j in STAGES} binary , >= 0 , <= 1 ;

##### OBJECTIVE
minimize cost:

	sum{j in STAGES1} v1[j]
	+ sum{j in STAGES2} ( v1[j] + v3[j] )
	+ sum{j in STAGES2 inter UNIT2} (v2[j])
	+ sum{j in STAGES3 union STAGES4} (v1[j] + v2[j] + v3[j])
	+ r*RHO*coef_delta ;

##### CONSTRAINTS
## Restricciones para etapas batch
subject to C1_BATCH{(i,h,j) in PROUTES: j in STAGES1}:
	y1[j] + x1[j]	>=  s1[i,h,j]*z1[i,h] + y4[i,h] ;
subject to C2_BATCH{(i,h,j) in PROUTES: j in STAGES1}:
	y5[i,h] + x2[j]	>=  t0[i,h,j]*z1[i,h] ;

## Restricciones para unidades semicontinuas
subject to C1_SEMIC{(i,h,j) in PROUTES: j in STAGES2}:
	y1[j]	>=  s1[i,h,j]*z1[i,h] + y4[i,h] ;
subject to C2_SEMIC{(i,h,j) in PROUTES: (h,j) in ROUTES2}:
	y2[j]	>=  s2[i,h,j]*z1[i,h] + y4[i,h] ;
subject to C3_SEMIC{(i,h,j) in PROUTES: j in STAGES2}:
	y5[i,h] + x1[j] + x2[j] + y3[j]	>=  t1[i,h,j]*z1[i,h] + y4[i,h] ;

### Restricciones para etapas cromatograficas
subject to C1_CHROM{(i,h,j) in PROUTES: j in STAGES5}:
	y1[j] >=  s1[i,h,j]*z1[i,h] + y4[i,h];
subject to C2_CHROM{(i,h,j) in PROUTES: j in STAGES5}:
	y2[j] >=  s2[i,h,j]*z1[i,h] + y4[i,h] ;
subject to C3_CHROM{(i,h,j) in PROUTES: j in STAGES5}:
	y3[j] + x1[j] >=  s3[i,h,j]*z1[i,h] + y4[i,h] ;
subject to C5_CHROM{(i,h,j) in PROUTES: j in STAGES4}:
	y5[i,h] + x2[j] >=  t0[i,h,j]*z1[i,h] ;

## Horizonte de planificacion
subject to CHP:
	sum{(i,h) in HP} v7[i,h] <= (delta/coef_delta) + r ;

## Variables binarias para numero de etapas en paralelo
subject to EP1{j in STAGES}:
	x1[j] = sum{k in UNITSG} y6[j,k]*log(k) ;
subject to EP2{j in STAGES}:
	sum{k in UNITSG} y6[j,k] = z2[j] ;
subject to EP3{j in STAGES}:
	x2[j] = sum{k in UNITSM} y7[j,k]*log(k) ;
subject to EP4{j in STAGES}:
	sum{k in UNITSM} y7[j,k] = z2[j] ;

subject to VB1{i in PRODUCTS}:
	sum{h in HOSTS: (i,h) in HP} z1[i,h] = 1 ;
subject to VB2{(i,h,j) in PROUTES: (i,h) in HP}:
	z2[j] >= z1[i,h] ;

subject to UB1{j in STAGES}:	y1up[j]*z2[j] >= y1[j] ;
subject to UB2{j in UNIT2}:		y2up[j]*z2[j] >= y2[j] ;
subject to UB3{j in UNIT3}:		y3up[j]*z2[j] >= y3[j] ;
subject to UB4{(i,h) in HP}:	y4up[i,h]*z1[i,h] >= y4[i,h] ;
subject to UB5{(i,h) in HP}:	y5up[i,h]*z1[i,h] >= y5[i,h] ;
subject to LB1{j in STAGES}:	y1[j] >= y1lo[j]*z2[j] ;
subject to LB2{j in UNIT2}:		y2[j] >= y2lo[j]*z2[j] ;
subject to LB3{j in UNIT3}:		y3[j] >= y3lo[j]*z2[j] ;
subject to LB4{(i,h) in HP}:	y4[i,h] >= y4lo[i,h]*z1[i,h] ;
subject to LB5{(i,h) in HP}:	y5[i,h] >= y5lo[i,h]*z1[i,h] ;

subject to UB6{j in STAGES}:	log(NM)*z2[j]	>= x1[j] ;
subject to UB7{j in STAGES}:	log(NG)*z2[j]	>= x2[j] ;
subject to LB6{j in STAGES}:	x1[j]	>= log(1)*z2[j] ;
subject to LB7{j in STAGES}:	x2[j]	>= log(1)*z2[j] ;

subject to UB8{j in STAGES}:	v1[j] <= v1up[j]*z2[j] ;
subject to UB9{j in UNIT2}:		v2[j] <= v2up[j]*z2[j] ;
subject to UB10{j in UNIT3}:	v3[j] <= v3up[j]*z2[j] ;
subject to UB11{(i,h) in HP}:	v7[i,h] <= v7up[i,h]*z1[i,h] ;

## funcion no lineal FO
subject to C1v1{j in STAGES1, k in TMIP1U}:
	v1[j] >= alpha1[j,k]*(x1[j]+x2[j]+gamma1[j]*y1[j]) - alpha1[j,k]*b1[j,k]*z2[j] + beta1[j,k]*z2[j] ;
subject to C2v1{j in UNIT1 diff STAGES1, k in TMIP1U}:
	v1[j] >= alpha1[j,k]*(x2[j]+gamma1[j]*y1[j]) - alpha1[j,k]*b1[j,k]*z2[j] + beta1[j,k]*z2[j] ;
subject to C1v2{j in UNIT2, k in TMIP2U}:
	v2[j] >= alpha2[j,k]*(x2[j]+gamma2[j]*y2[j]) - alpha2[j,k]*b2[j,k]*z2[j] + beta2[j,k]*z2[j] ;
subject to C1v3{j in UNIT3, k in TMIP3U}:
	v3[j] >= alpha3[j,k]*(x1[j]+x2[j]+gamma3[j]*y3[j]) - alpha3[j,k]*b3[j,k]*z2[j] + beta3[j,k]*z2[j] ;
subject to C1v7{(i,h) in HP,k in TMIP7U}:
	v7[i,h] >= alpha7[i,h,k]*(y5[i,h]-y4[i,h]) - alpha7[i,h,k]*b7[i,h,k]*z1[i,h] + beta7[i,h,k]*z1[i,h] ;
## funcion no lineal cromatografica
subject to C4_CHROM{(i,h,j) in PROUTES, k in TMIP6U: j in STAGES3}:
	y5[i,h] + x2[j] >=  alpha6[i,h,j,k]*(y4[i,h]-x1[j]-y3[j]) -alpha6[i,h,j,k]*b6[i,h,j,k]*z1[i,h] + beta6[i,h,j,k]*z1[i,h] ;
